package com.ruoyi.wx.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.wx.domain.MrRecordOrder;
import com.ruoyi.wx.service.IMrRecordOrderService;
import com.ruoyi.wx.util.ShortUrlUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 病案申请Controller
 *
 * @author ruoyi
 * @date 2021-02-24
 */
@Controller
@RequestMapping("/wx/order")
public class MrRecordOrderController extends BaseController {
    private String prefix = "wx/order";

    @Value("${server.servlet.domain}")
    private String domain;

    @Autowired
    private IMrRecordOrderService mrRecordOrderService;

    @RequiresPermissions("wx:order:view")
    @GetMapping()
    public String order() {
        return prefix + "/order";
    }

    /**
     * 查询病案申请列表
     */
    @RequiresPermissions("wx:order:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(MrRecordOrder mrRecordOrder) {
        startPage();
        List<MrRecordOrder> list = mrRecordOrderService.selectMrRecordOrderList(mrRecordOrder);
        return getDataTable(list);
    }

    /**
     * 导出病案申请列表
     */
    @RequiresPermissions("wx:order:export")
    @Log(title = "病案申请", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(MrRecordOrder mrRecordOrder) {
        List<MrRecordOrder> list = mrRecordOrderService.selectMrRecordOrderList(mrRecordOrder);
        ExcelUtil<MrRecordOrder> util = new ExcelUtil<MrRecordOrder>(MrRecordOrder.class);
        return util.exportExcel(list, "order");
    }

    /**
     * 新增病案申请
     */
    @GetMapping("/add")
    public String add() {
        return prefix + "/add";
    }

    /**
     * 新增保存病案申请
     */
    @RequiresPermissions("wx:order:add")
    @Log(title = "病案申请", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(MrRecordOrder mrRecordOrder) {
        return toAjax(mrRecordOrderService.insertMrRecordOrder(mrRecordOrder));
    }

    /**
     * 修改病案申请
     */
    @GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") Long id, ModelMap mmap) {
        MrRecordOrder mrRecordOrder = mrRecordOrderService.selectMrRecordOrderById(id);
        mmap.put("mrRecordOrder", mrRecordOrder);
        return prefix + "/edit";
    }

    /**
     * 修改保存病案申请
     */
    @RequiresPermissions("wx:order:edit")
    @Log(title = "病案申请", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(MrRecordOrder mrRecordOrder) {
        return toAjax(mrRecordOrderService.updateMrRecordOrder(mrRecordOrder));
    }

    /**
     * 删除病案申请
     */
    @RequiresPermissions("wx:order:remove")
    @Log(title = "病案申请", businessType = BusinessType.DELETE)
    @PostMapping("/remove")
    @ResponseBody
    public AjaxResult remove(String ids) {
        return toAjax(mrRecordOrderService.deleteMrRecordOrderByIds(ids));
    }

    @GetMapping("/sendView/{id}")
    public String sendView(@PathVariable("id") Long id, ModelMap mmap) {
        MrRecordOrder mrRecordOrder = mrRecordOrderService.selectMrRecordOrderById(id);
        mmap.put("mrRecordOrder", mrRecordOrder);
        mmap.put("randomCode", RandomStringUtils.random(8,true,true));
        return prefix + "/sendView";
    }

    @Log(title = "发送病案处理短信", businessType = BusinessType.UPDATE)
    @PostMapping("/sendSms")
    @ResponseBody
    public AjaxResult sendSms(MrRecordOrder mrRecordOrder) {
        return toAjax(mrRecordOrderService.sendSms(mrRecordOrder));
    }

    @PostMapping("/upload")
    @ResponseBody
    public AjaxResult upload(@RequestParam("file_data") MultipartFile file) throws IOException {
        // 上传文件路径
        String filePath = RuoYiConfig.getUploadPath();
        // 上传并返回新文件名称
        String fileName = FileUploadUtils.upload(filePath, file);
        // 保存短网址对应关系
        String code=mrRecordOrderService.randomCode(fileName,file.getOriginalFilename());
        return AjaxResult.success("上传成功", domain+"/s/"+code);
    }
}
