package com.ruoyi.wx.service.impl;

import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.BusinessException;
import com.ruoyi.common.exception.wx.WxBusinessException;
import com.ruoyi.common.json.JSONObject;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.html.EscapeUtil;
import com.ruoyi.wx.domain.MrRecordOrder;
import com.ruoyi.wx.domain.WxUser;
import com.ruoyi.wx.dto.RecordOrderDTO;
import com.ruoyi.wx.dto.WxUserDTO;
import com.ruoyi.wx.dto.req.OrderListReqDTO;
import com.ruoyi.wx.mapper.MrRecordOrderMapper;
import com.ruoyi.wx.mapper.WxUserMapper;
import com.ruoyi.wx.service.IMrRecordOrderService;
import com.ruoyi.wx.util.SmsUtils;
import com.ruoyi.wx.util.WxServiceUtils;
import com.ruoyi.wx.util.WxUserUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 病案申请Service业务层处理
 *
 * @author ruoyi
 * @date 2021-02-24
 */
@Service
public class MrRecordOrderServiceImpl implements IMrRecordOrderService {
    @Autowired
    private MrRecordOrderMapper mrRecordOrderMapper;

    /**
     * 查询病案申请
     *
     * @param id 病案申请ID
     * @return 病案申请
     */
    @Override
    public MrRecordOrder selectMrRecordOrderById(Long id) {
        return mrRecordOrderMapper.selectMrRecordOrderById(id);
    }

    /**
     * 查询病案申请列表
     *
     * @param mrRecordOrder 病案申请
     * @return 病案申请
     */
    @Override
    public List<MrRecordOrder> selectMrRecordOrderList(MrRecordOrder mrRecordOrder) {
        return mrRecordOrderMapper.selectMrRecordOrderList(mrRecordOrder);
    }

    /**
     * 新增病案申请
     *
     * @param mrRecordOrder 病案申请
     * @return 结果
     */
    @Override
    public int insertMrRecordOrder(MrRecordOrder mrRecordOrder) {
        mrRecordOrder.setCreateTime(DateUtils.getNowDate());
        return mrRecordOrderMapper.insertMrRecordOrder(mrRecordOrder);
    }

    /**
     * 修改病案申请
     *
     * @param mrRecordOrder 病案申请
     * @return 结果
     */
    @Override
    public int updateMrRecordOrder(MrRecordOrder mrRecordOrder) {
        mrRecordOrder.setUpdateTime(DateUtils.getNowDate());
        return mrRecordOrderMapper.updateMrRecordOrder(mrRecordOrder);
    }

    /**
     * 删除病案申请对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteMrRecordOrderByIds(String ids) {
        return mrRecordOrderMapper.deleteMrRecordOrderByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除病案申请信息
     *
     * @param id 病案申请ID
     * @return 结果
     */
    @Override
    public int deleteMrRecordOrderById(Long id) {
        return mrRecordOrderMapper.deleteMrRecordOrderById(id);
    }

    @Autowired
    private WxUserUtils wxUserUtils;
    @Autowired
    private WxUserMapper wxUserMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createOrder(MrRecordOrder order) {
        // 判断参数是否遗漏
        if (StringUtils.isBlank(order.getMrNumber())) {
            throw new WxBusinessException("病案号必填");
        }
        if (StringUtils.isBlank(order.getPatientName())) {
            throw new WxBusinessException("患者姓名必填");
        }
        if (Objects.isNull(order.getDischargeDate())) {
            throw new WxBusinessException("出院时间必填");
        }
        if (Objects.isNull(order.getTimes())) {
            throw new WxBusinessException("就诊次数必填");
        }
        if (order.getType() == 0) {
            if (Objects.isNull(order.getAddress())) {
                throw new WxBusinessException("配送地址必填");
            }
        }

        // 判断是否已有相同订单
        MrRecordOrder searchOrder = new MrRecordOrder();
        searchOrder.setMrNumber(order.getMrNumber());
        searchOrder.setType(order.getType());
        searchOrder.setStatus(0);
        List<MrRecordOrder> mrRecordOrders = mrRecordOrderMapper.selectMrRecordOrderList(searchOrder);
        if (mrRecordOrders != null && !mrRecordOrders.isEmpty()) {
            throw new WxBusinessException("已有相同申请！");
        }

        WxUserDTO userInfo = wxUserUtils.getUserInfo();
        order.setApplicantId(userInfo.getId());
        order.setHospital(userInfo.getHospital());

        if (StringUtils.isNotBlank(order.getAddress())) {
            WxUser wxUser = new WxUser();
            wxUser.setId(userInfo.getId());
            wxUser.setAddress(order.getAddress());
            wxUserMapper.updateWxUser(wxUser);
        }

        order.setCreateTime(new Date());
        order.setUpdateTime(new Date());
        order.setStatus(0);
        mrRecordOrderMapper.insertMrRecordOrder(order);
    }

    @Override
    public List<RecordOrderDTO> list(OrderListReqDTO orderListReqDTO) {
        int rowCount = 10;
        int startRow = rowCount * (orderListReqDTO.getPage() - 1);
        int endRow = rowCount * orderListReqDTO.getPage();
        return mrRecordOrderMapper.searchList(wxUserUtils.getUserInfo().getId(), orderListReqDTO.getKeyword(), orderListReqDTO.getStatus(), startRow, endRow);
    }

    @Override
    public boolean sendSms(MrRecordOrder mrRecordOrder) {
        MrRecordOrder mrRecordOrder1 = mrRecordOrderMapper.selectMrRecordOrderById(mrRecordOrder.getId());
        if (Objects.nonNull(mrRecordOrder1)) {
            WxUser wxUser = wxUserMapper.selectWxUserById(mrRecordOrder1.getApplicantId());
            // 发送短信
            if (mrRecordOrder.getType() == 1) {
                SmsUtils.sendSms(SmsUtils.TemplateType.DOWN, wxUser.getPhone(), mrRecordOrder.getUnzipPwd());
            } else if (mrRecordOrder.getType() == 0) {
                SmsUtils.sendSms(SmsUtils.TemplateType.Express, wxUser.getPhone(), mrRecordOrder.getTrackingNumber());
            }
            // 更新订单
            mrRecordOrder.setStatus(1);
            mrRecordOrder.setUpdateTime(new Date());
            mrRecordOrderMapper.updateMrRecordOrder(mrRecordOrder);
        }
        return true;
    }

    @Override
    public Map<String, String> getUrlHeaders(String code) {
        return mrRecordOrderMapper.getUrlHeaders(code);
    }

    @Override
    public String randomCode(String url, String filename) {
        return saveUrl(url, filename, 1);
    }

    private String saveUrl(String url, String filename, int times) {
        try {
            String code = RandomStringUtils.random(4, true, true);
            JSONObject jsonObject = new JSONObject();
            jsonObject.set("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, Charset.defaultCharset().displayName()));
            mrRecordOrderMapper.saveCodeUrl(code, url, jsonObject.toCompactString());
            return code;
        } catch (Exception e) {
            // code为唯一建，失败了就重试,最多重试5次
            if (times > 5) {
                throw new BusinessException("创建短网址失败");
            }
            return saveUrl(url, filename, ++times);
        }
    }
}