package com.ruoyi.wx.service.impl;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.BusinessException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.wx.domain.WxUser;
import com.ruoyi.wx.dto.WxUserDTO;
import com.ruoyi.wx.dto.req.UserSignInfo;
import com.ruoyi.wx.mapper.WxUserMapper;
import com.ruoyi.wx.service.IWxUserService;
import com.ruoyi.wx.util.WxServiceUtils;
import com.ruoyi.wx.util.WxUserUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 微信小程序用户Service业务层处理
 *
 * @author ruoyi
 * @date 2021-02-24
 */
@Service
public class WxUserServiceImpl implements IWxUserService {
    @Autowired
    private WxUserMapper wxUserMapper;
    @Autowired
    private WxServiceUtils wxServiceUtils;
    @Autowired
    private WxUserUtils wxUserUtils;

    /**
     * 查询微信小程序用户
     *
     * @param id 微信小程序用户ID
     * @return 微信小程序用户
     */
    @Override
    public WxUser selectWxUserById(Long id) {
        return wxUserMapper.selectWxUserById(id);
    }

    /**
     * 查询微信小程序用户列表
     *
     * @param wxUser 微信小程序用户
     * @return 微信小程序用户
     */
    @Override
    public List<WxUser> selectWxUserList(WxUser wxUser) {
        return wxUserMapper.selectWxUserList(wxUser);
    }

    /**
     * 新增微信小程序用户
     *
     * @param wxUser 微信小程序用户
     * @return 结果
     */
    @Override
    public int insertWxUser(WxUser wxUser) {
        wxUser.setCreateTime(DateUtils.getNowDate());
        return wxUserMapper.insertWxUser(wxUser);
    }

    /**
     * 修改微信小程序用户
     *
     * @param wxUser 微信小程序用户
     * @return 结果
     */
    @Override
    public int updateWxUser(WxUser wxUser) {
        wxUser.setUpdateTime(DateUtils.getNowDate());
        return wxUserMapper.updateWxUser(wxUser);
    }

    /**
     * 删除微信小程序用户对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteWxUserByIds(String ids) {
        return wxUserMapper.deleteWxUserByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除微信小程序用户信息
     *
     * @param id 微信小程序用户ID
     * @return 结果
     */
    @Override
    public int deleteWxUserById(Long id) {
        return wxUserMapper.deleteWxUserById(id);
    }

    @Override
    public WxUser selectWxUserByToken(String token) {
        return wxUserMapper.selectWxUserByToken(token);
    }

    @Override
    public WxUserDTO login(String wxCode) {
        WxUserDTO wxUserDTO = new WxUserDTO();
        WxMaJscode2SessionResult login = wxServiceUtils.login(wxCode);
        // 根据openid查询用户
        WxUser wxUser = wxUserMapper.selectWxUserByOpenid(login.getOpenid());

        if (Objects.nonNull(wxUser)) {
            wxUser.setToken(login.getSessionKey());
            wxUser.setUpdateTime(new Date());
            wxUserMapper.updateWxUser(wxUser);
        } else {
            wxUser = new WxUser();
            wxUser.setToken(login.getSessionKey());
            wxUser.setOpenid(login.getOpenid());
            wxUser.setStatus(2);
            wxUser.setFlag(0);
            wxUser.setCreateTime(new Date());
            wxUserMapper.insertWxUser(wxUser);
        }
        BeanUtils.copyProperties(wxUser, wxUserDTO);
        return wxUserDTO;
    }

    @Override
    public void signIn(UserSignInfo userSignInfo) {
        WxUserDTO userInfo = wxUserUtils.getUserInfo();

        WxUser wxUser = wxUserMapper.selectWxUserById(userInfo.getId());
        wxUser.setUpdateTime(new Date());
        wxUser.setAvatar(userSignInfo.getAvatar());
        wxUser.setNickname(userSignInfo.getNickName());
        String gender = "未知";
        if (StringUtils.equals("1", userSignInfo.getGender())) {
            gender = "男";
        } else if (StringUtils.equals("2", userSignInfo.getGender())) {
            gender = "女";
        }
        wxUser.setGender(gender);
        wxUser.setHospital(userSignInfo.getHospital());
        wxUser.setName(userSignInfo.getName());
        wxUser.setPhone(userSignInfo.getPhone());

        wxUser.setStatus(0);

        wxUserMapper.updateWxUser(wxUser);
        wxUserUtils.clearCache();
    }

    @Override
    public WxUserDTO info(UserSignInfo userSignInfo) {
        WxUserDTO userInfo = wxUserUtils.getUserInfo();

        WxUser wxUser = wxUserMapper.selectWxUserById(userInfo.getId());
        if (!wxUser.getStatus().equals(userInfo.getStatus()) || !wxUser.getFlag().equals(userInfo.getFlag())) {// 状态变更时更新用户信息
            wxUser.setUpdateTime(new Date());
            wxUser.setAvatar(userSignInfo.getAvatar());
            wxUser.setNickname(userSignInfo.getNickName());
            String gender = "未知";
            if (StringUtils.equals("1", userSignInfo.getGender())) {
                gender = "男";
            } else if (StringUtils.equals("2", userSignInfo.getGender())) {
                gender = "女";
            }
            wxUser.setGender(gender);

            wxUserMapper.updateWxUser(wxUser);
            wxUserUtils.clearCache();
            BeanUtils.copyProperties(wxUser, userInfo);
        }

        return userInfo;
    }

    @Override
    public boolean checkUser(Long id) {
        WxUser wxUser=new WxUser();
        wxUser.setId(id);
        wxUser.setFlag(1);
        updateWxUser(wxUser);
        return true;
    }

    @Override
    public boolean triggerStatus(Long id) {
        WxUser wxUser=selectWxUserById(id);
        if(wxUser.getStatus()==1){
            wxUser.setStatus(0);
            updateWxUser(wxUser);
        }else if(wxUser.getStatus()==0){
            wxUser.setStatus(1);
            updateWxUser(wxUser);
        }
        return true;
    }

    @Override
    public List<WxUser> searchByName(String uname) {
        WxUser wxUser=new WxUser();
        wxUser.setName(uname);
        return wxUserMapper.selectWxUserList(wxUser);
    }
}
