package com.ruoyi.wx.util;

import com.aliyun.dysmsapi20170525.Client;
import com.aliyun.dysmsapi20170525.models.SendSmsRequest;
import com.aliyun.dysmsapi20170525.models.SendSmsResponse;
import com.aliyun.teaopenapi.models.Config;
import com.ruoyi.common.exception.BusinessException;
import com.ruoyi.common.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

@Component
public class SmsUtils {
    private static final Logger log = LoggerFactory.getLogger(SmsUtils.class);

    private static String accessKeyId;
    private static String accessKeySecret;
    private static String signName;

    @Value("${aliyun.accesskey-id}")
    String aliAccessKeyId;
    @Value("${aliyun.accesskey-secret}")
    String aliAccessKeySecret;
    @Value("${aliyun.sms.sign-name}")
    String pSignName;

    @PostConstruct
    private void initialization() {
        accessKeyId = aliAccessKeyId;
        accessKeySecret = aliAccessKeySecret;
        signName = pSignName;
    }

    /**
     * 使用AK&SK初始化账号Client
     *
     * @return Client
     * @throws Exception
     */
    public static Client getClient() throws Exception {
        Config config = new Config()
                // 您的AccessKey ID
                .setAccessKeyId(accessKeyId)
                // 您的AccessKey Secret
                .setAccessKeySecret(accessKeySecret);
        // 访问的域名
        config.endpoint = "dysmsapi.aliyuncs.com";
        return new Client(config);
    }

    /**
     * 发送短信
     *
     * @param type      模板类型
     * @param telephone 手机号码
     * @param objs      参数值数组，对应模板参数数组传入
     */
    public static void sendSms(TemplateType type, String telephone, Object... objs) {
        // 组装请求对象-具体描述见控制台-文档部分内容
        SendSmsRequest request = new SendSmsRequest();
        // 必填:待发送手机号
        request.setPhoneNumbers(telephone);
        //必填:短信签名-可在短信控制台中找到
        request.setSignName(signName);
        // 必填:短信模板-可在短信控制台中找到
        request.setTemplateCode(type.code);
        if (type.args != null && type.args.length > 0) {
            JSONObject jsonObject = new JSONObject();
            for (int i = 0; i < type.args.length; i++) {
                jsonObject.put(type.args[i], objs[i]);
            }
            request.setTemplateParam(jsonObject.toCompactString());
        }
        //请求失败这里会抛ClientException异常
        SendSmsResponse sendSmsResponse = null;
        try {
            sendSmsResponse = getClient().sendSms(request);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (sendSmsResponse != null && "OK".equals(sendSmsResponse.getBody().getCode())) {
            log.debug("send Message to [{}] successful", telephone);
        } else {
            if (null != sendSmsResponse) {
                throw new BusinessException("发送短信失败," + sendSmsResponse.getBody().getMessage());
            } else {
                throw new BusinessException("发送短信失败");
            }
        }
    }

    public enum TemplateType {
        /**
         * 下载扫描件通知模板
         */
        DOWN("SMS_212471722", "pwd"),
        /**
         * 快递单通知模板
         */
        Express("SMS_212476711", "nu");
        /**
         * 参数名称数组
         */
        private final String[] args;
        /**
         * 模板id
         */
        private final String code;

        TemplateType(String code, String... args) {
            this.args = args;
            this.code = code;
        }
    }
}
