package com.ruoyi.wx.util;

import com.ruoyi.common.exception.wx.NotLoginException;
import com.ruoyi.common.utils.CacheUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.wx.domain.WxUser;
import com.ruoyi.wx.dto.WxUserDTO;
import com.ruoyi.wx.service.IWxUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

@Component
public class WxUserUtils {
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private IWxUserService wxUserService;
    private final String cacheName = "wx-userCache";

    private String getToken() {
        return request.getHeader("token");
    }

    public WxUserDTO getUserInfo() {
        String token = getToken();
        Object userCache = CacheUtils.get(cacheName, "wx-token:" + token);
        if (Objects.nonNull(userCache)) {
            return (WxUserDTO) userCache;
        } else {
            WxUser wxUser = wxUserService.selectWxUserByToken(getToken());
            if (Objects.nonNull(wxUser)) {
                WxUserDTO wxUserDTO = new WxUserDTO();
                BeanUtils.copyProperties(wxUser, wxUserDTO);
                CacheUtils.put(cacheName, "wx-token:" + token, wxUserDTO);
                return wxUserDTO;
            } else {
                throw new NotLoginException();
            }
        }
    }

    public void clearCache() {
        CacheUtils.remove(cacheName, "wx-token:" + getToken());
    }
}
